# Copyright 2005-2007, Google, Inc.

# This software is provided as an example of using the Ruby interface
# to SketchUp.

# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------

require 'sketchup.rb'
require 'langhandler.rb'

$exStrings = LanguageHandler.new("CLineTool")

class CLineTool
  @@ltcursor = 0
  @@ltcursor_p = 0

  def initialize
    @ip1 = nil
    @ip2 = nil
    @xdown = 0
    @ydown = 0
    @draw_cpoints = true
	dir=File.dirname(__FILE__).split('/')[0..-2].join('/')+"/icones"
    if @@ltcursor == 0
      c = File.join(dir, "clinetool", "clinetool_cursor.png")
      @@ltcursor = UI::create_cursor(c, 1, 30)
    end
    if @@ltcursor_p == 0
      c = File.join(dir, "clinetool", "clinetool_cursor_p.png")
      @@ltcursor_p = UI::create_cursor(c, 1, 30)
    end
  end

  def onSetCursor
    if @draw_cpoints
      UI::set_cursor(@@ltcursor_p)
    else
      UI::set_cursor(@@ltcursor)
    end
  end

  def activate
    @ip1 = Sketchup::InputPoint.new
    @ip2 = Sketchup::InputPoint.new
    @ip = Sketchup::InputPoint.new
    @drawn = false
    @last_drawn = nil
    Sketchup::set_status_text $exStrings.GetString("Length"), SB_VCB_LABEL
    if Sketchup.active_model.rendering_options["HideConstructionGeometry"] == true 
      Sketchup.active_model.rendering_options["HideConstructionGeometry"] = false 
    end

    self.reset(nil)
  end

  def resume(view)
    txt = "Construction Line Tool: CTRL = toggle points."
    Sketchup::set_status_text $exStrings.GetString(txt), SB_PROMPT
  end

  def deactivate(view)
    view.invalidate if @drawn
  end

  def onMouseMove(flags, x, y, view)
    if( @state == 0 )
		@ip.pick view, x, y
		if( @ip != @ip1 )
			view.invalidate if( @ip.display? or @ip1.display? )
			@ip1.copy! @ip
			view.tooltip = @ip1.tooltip
		end
    else
		@ip2.pick view, x, y, @ip1
		view.tooltip = @ip2.tooltip if( @ip2.valid? )
		view.invalidate
		if( @ip2.valid? )
			length = @ip1.position.distance(@ip2.position)
			Sketchup::set_status_text length.to_s, SB_VCB_VALUE
		end
		if( (x-@xdown).abs > 10 || (y-@ydown).abs > 10 )
		@dragging = true
		end
    end
  end

  def onLButtonDown(flags, x, y, view)
    if( @state == 0 )
      @ip1.pick view, x, y
      if( @ip1.valid? )
	@state = 1
	txt = "Select second end. Ctrl = toggle points "
	if @draw_cpoints
	  txt << "off."
	else
	  txt << "on."
	end
	Sketchup::set_status_text $exStrings.GetString(txt), SB_PROMPT
	@xdown = x
	@ydown = y
      end
    else
      if( @ip2.valid? )
	self.create_geometry(@ip1.position, @ip2.position,view)
	self.reset(view)
      end
    end
    view.lock_inference
  end

  def onLButtonUp(flags, x, y, view)
    if( @dragging && @ip2.valid? )
      self.create_geometry(@ip1.position, @ip2.position,view)
      self.reset(view)
    end
  end

  def onKeyDown(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY && repeat == 1 )
      @shift_down_time = Time.now
      if( view.inference_locked? )
	view.lock_inference
      elsif( @state == 0 && @ip1.valid? )
	view.lock_inference @ip1
	view.line_width = 3
      elsif( @state == 1 && @ip2.valid? )
	view.lock_inference @ip2, @ip1
	view.line_width = 3
      end
    end
    if( key == COPY_MODIFIER_KEY )
      @draw_cpoints = !@draw_cpoints
      if @state == 0
	txt = "Select first end. Ctrl = toggle points "
      else
	txt = "Select second end. Ctrl = toggle points "
      end
      if @draw_cpoints
	txt << "off."
      else
	txt << "on."
      end
      Sketchup::set_status_text $exStrings.GetString(txt), SB_PROMPT
    end
  end

  def onKeyUp(key, repeat, flags, view)
    if( key == CONSTRAIN_MODIFIER_KEY &&
       view.inference_locked? &&
       (Time.now - @shift_down_time) > 0.5 )
      view.lock_inference
    end
  end
  def onUserText(text, view)
    begin
      value = text.to_l
    rescue
      UI.beep
      puts "Cannot convert #{text} to a Length"
      value = nil
      Sketchup::set_status_text "", SB_VCB_VALUE
    end
    return if !value

    if @state == 1 and @ip2.valid?
      pt1 = @ip1.position
      vec = @ip2.position - pt1
      if( vec.length == 0.0 )
	UI.beep
	return
      end
      vec.length = value
      pt2 = pt1 + vec
      self.create_geometry(pt1, pt2, view)
      self.reset(view)
    end
    if @last_drawn and @state == 0
      pt1 = @last_drawn[0].start
      vec = @last_drawn[0].end - @last_drawn[0].start
      vec.length = value
      pt2 = pt1 + vec
      view.model.active_entities.erase_entities @last_drawn
      self.create_geometry(pt1, pt2, view)
      self.reset view
    end
  end
  
  def draw(view)
    if( @ip1.valid? )
      if( @ip1.display? )
	@ip1.draw(view)
	@drawn = true
      end

      if( @ip2.valid? )
	@ip2.draw(view) if( @ip2.display? )
	view.set_color_from_line(@ip1, @ip2)
	self.draw_geometry(@ip1.position, @ip2.position, view)
	@drawn = true
      end
    end
  end

  def onCancel(flag, view)
    self.reset(view)
  end

  def reset(view)
    @state = 0

    txt = "Select first end. Ctrl = toggle points "
    if @draw_cpoints
      txt << "off."
    else
      txt << "on."
    end
    Sketchup::set_status_text($exStrings.GetString(txt), SB_PROMPT)

    @ip1.clear
    @ip2.clear
    @ip.clear

    if( view )
      view.tooltip = nil
      view.invalidate if @drawn
    end

    @drawn = false
    @dragging = false
  end

  def create_geometry(p1, p2, view)
    Sketchup.active_model.start_operation("CLineTool")
    @layer = Sketchup.active_model.layers.add("CLineTool")
    cline = view.model.active_entities.add_cline(p1,p2)
    cline.layer = @layer
    @last_drawn = [cline]
    if @draw_cpoints
      cpoint =  view.model.active_entities.add_cpoint(p1)
      cpoint.layer = @layer
      @last_drawn << cpoint
      cpoint =  view.model.active_entities.add_cpoint(p2)
      cpoint.layer = @layer
      @last_drawn << cpoint
    end
    Sketchup.active_model.commit_operation
  end

  def draw_geometry(pt1, pt2, view)
    view.draw_line(pt1, pt2)
  end

  def set_text
  end

end # class LineTool
